@@ -1,50 +1,51 @@
-/* Redmond Engine
+/* Redmond95 - a cairo based GTK+ engine
  * Copyright (C) 2001 Red Hat, Inc.
+ * Copyright (C) 2006 Andrew Johnson <acjgenius@earthlink.net>
+ * Copyright (C) 2006-2007 Benjamin Berg <benjamin@sipsolutions.net>
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
+ * version 2.1 of the License, or (at your option) any later version.
+ * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+ * Project contact: <gnome-themes-list@gnome.org>
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
  *
- * Written by Owen Taylor <otaylor@redhat.com>
- * modified by Andrew Johnson <ajgenius@ajgenius.us>
- */
-#include "redmond_gtk2_engine.h"
-#include "redmond_gtk2_drawing.h"
-#include "redmond_gtk2_misc.h"
- 
-/***********************************************************************
  * Portions of the code based on/copied from the Bluecurve theme/engine, 
- *   License: GPL 
+ *   License: LGPL2+
  *   Copyright:
  *     Alexander Larsson <alexl@redhat.com>
  *     Garrett LeSage
  *     Owen Taylor <otaylor@redhat.com>
  *
  * Portions of the code based on/copied from the Metal theme/engine, 
- *   License: GPL 
+ *   License: LGPL2.1+
  *   Copyright:
  *     Randy Gordon, randy@integrand.com
  *     Owen Taylor <otaylor@redhat.com>
  *
  * Portions of the code based on/copied from the Smooth Engine,
- *   License: LGPL 
+ *   License: LGPL
  *   Copyright:
  *     Andrew Johnson <ajgenius@ajgenius.us
  *
- ***********************************************************************/
- 
- 
+ */
+
+
+#include "redmond_gtk2_engine.h"
+#include "redmond_gtk2_drawing.h"
+#include "redmond_gtk2_misc.h"
+
+
 /***********************************************
  * redmond_draw_vline -
  *  
@@ -153,64 +154,55 @@ redmond_draw_check (GtkStyle * style,
  
   cr = ge_gdk_drawable_to_cairo (window, area);
 
-  if (CHECK_DETAIL (detail, "check"))	/* Menu item */
+  if (CHECK_DETAIL (detail, "check") && /* Special case for menu items. */
+     (shadow != GTK_SHADOW_ETCHED_IN))
     {
-      /* check boxes in menu don't actually draw box, only the check mark,
-       * which use the text color 
-       */
       if (shadow == GTK_SHADOW_IN)
-	{
-	  do_redmond_draw_check (cr, &redmond_style->color_cube.text[state], x + 3, y + 3,
-			 width - 4, height - 4);
-	}
+        do_redmond_draw_check (cr, &redmond_style->color_cube.text[GTK_STATE_NORMAL],
+                               x + 2, y + 2, width - 4, height - 4);
     }
   else
     {
-      switch (state)
-	{
-	case GTK_STATE_NORMAL:
-	case GTK_STATE_PRELIGHT:
-          if (IS_TOGGLE_BUTTON(widget) && TOGGLE_BUTTON(widget)->inconsistent)
-            do_redmond_draw_masked_fill (cr, &redmond_style->hatch_mask, 
-                                              &redmond_style->color_cube.bg[GTK_STATE_NORMAL],
-                                              &redmond_style->color_cube.light[GTK_STATE_NORMAL],
-                                              x, y, width, height);
-	  else
+      if (state == GTK_STATE_ACTIVE || state == GTK_STATE_INSENSITIVE)
+      {
+        ge_cairo_set_color(cr, &redmond_style->color_cube.bg[GTK_STATE_NORMAL]);
+
+        cairo_rectangle(cr, x, y, width - 1, height - 1);
+
+        cairo_fill(cr);
+      }
+      else
+      {
+        /* Draw stipples for inconsistent checkboxes. */
+        if (shadow == GTK_SHADOW_ETCHED_IN)
           {
+            do_redmond_draw_masked_fill (cr, &redmond_style->hatch_mask, 
+                                             &redmond_style->color_cube.bg[GTK_STATE_NORMAL],
+                                             &redmond_style->color_cube.light[GTK_STATE_NORMAL],
+                                             x, y, width, height);
+          }
+        else
+          {
+            /* ignore PRELIGHT state ... */
             ge_cairo_set_color(cr, &redmond_style->color_cube.base[GTK_STATE_NORMAL]);
 
             cairo_rectangle(cr, x, y, width - 1, height - 1);
 
             cairo_fill(cr);
           }
+      }
 
-	  break;
-	default:
-          ge_cairo_set_color(cr, &redmond_style->color_cube.bg[GTK_STATE_NORMAL]);
-
-          cairo_rectangle(cr, x, y, width - 1, height - 1);
-
-          cairo_fill(cr);
-
-	}
- 
-      if ((shadow == GTK_SHADOW_IN) || 
-          (IS_TOGGLE_BUTTON(widget) && 
-           TOGGLE_BUTTON(widget)->inconsistent))
-	{
-	  if ((state == GTK_STATE_INSENSITIVE) || 
-              (IS_TOGGLE_BUTTON(widget) && 
-               TOGGLE_BUTTON(widget)->inconsistent))
-            {
-	      do_redmond_draw_check (cr, &redmond_style->color_cube.fg[GTK_STATE_INSENSITIVE],
-			             x + 2, y + 2, width - 4, height - 4);
-            }
-	  else
-            {
-	      do_redmond_draw_check (cr, &redmond_style->color_cube.fg[GTK_STATE_NORMAL],
-			             x + 2, y + 2, width - 4, height - 4);
-            }
-	}
+      if ((shadow == GTK_SHADOW_ETCHED_IN) || (state == GTK_STATE_INSENSITIVE))
+        {
+          /* force insensitive color for inconsistent checkboxes */
+          do_redmond_draw_check (cr, &redmond_style->color_cube.fg[GTK_STATE_INSENSITIVE],
+                                 x + 2, y + 2, width - 4, height - 4);
+        }
+      else if (shadow == GTK_SHADOW_IN)
+        {
+          do_redmond_draw_check (cr, &redmond_style->color_cube.fg[GTK_STATE_NORMAL],
+                                 x + 2, y + 2, width - 4, height - 4);
+        }
  
       redmond_draw_shadow (style, window, GTK_STATE_NORMAL, GTK_SHADOW_IN,
 			area, widget, detail, x, y, width, height);
@@ -251,99 +243,92 @@ redmond_draw_option (GtkStyle * style,
              gint height)
 {
   RedmondStyle *redmond_style = REDMOND_STYLE (style);
- 
+  gdouble bullet_radius;
   gint center_x;
   gint center_y;
   gint radius;
-
+ 
   cairo_t *cr;
 
   CHECK_ARGS
   SANITIZE_SIZE
- 
-  center_x = x + floor(width/2);
-  center_y = y + floor(height/2);
-  radius = floor(MIN(width, height)/2) + 1;
+
+  center_x = x + width/2;
+  center_y = y + height/2;
+  radius = MAX(MIN(width, height)/2, 6);
+  bullet_radius = MAX((radius - 2) * 0.50, 1);
 
   cr = ge_gdk_drawable_to_cairo (window, area);
 
-  if (CHECK_DETAIL (detail, "option"))	/* Menu item */
+  if (CHECK_DETAIL (detail, "check") && /* Special case for menu items. */
+     (shadow != GTK_SHADOW_ETCHED_IN))
     {
-      /* check boxes in menu don't actually draw box, only the mark,
-       * which use the text color 
-       */
       if (shadow == GTK_SHADOW_IN)
-      {
-        ge_cairo_set_color(cr, &redmond_style->color_cube.text[state]);	
-	cairo_arc(cr, center_x, center_y, radius*0.38, 0, 2 * M_PI);
-        cairo_fill(cr);
-      }
+        {
+          ge_cairo_set_color(cr, &redmond_style->color_cube.text[GTK_STATE_NORMAL]);
+          cairo_arc(cr, center_x, center_y, bullet_radius, 0, 2 * G_PI);
+          cairo_fill(cr);
+        }
     }
   else
     {
-      gboolean inconsistent;
+      do_redmond_draw_simple_circle (cr, &redmond_style->color_cube.dark[GTK_STATE_NORMAL],
+                                     &redmond_style->color_cube.light[GTK_STATE_NORMAL],
+                                     center_x, center_y, radius);
 
-  do_redmond_draw_simple_circle (cr, &redmond_style->color_cube.dark[GTK_STATE_NORMAL],
-				&redmond_style->color_cube.light[GTK_STATE_NORMAL],
-					center_x, 
-					center_y, 
-					radius);
+      do_redmond_draw_simple_circle (cr, &redmond_style->black_border[GTK_STATE_NORMAL],
+                                     &redmond_style->color_cube.bg[GTK_STATE_NORMAL],
+                                     center_x, center_y, radius - 1);
 
-  do_redmond_draw_simple_circle (cr, &redmond_style->black_border[GTK_STATE_NORMAL],
-				&redmond_style->color_cube.bg[GTK_STATE_NORMAL],
-					center_x, 
-					center_y, 
-					radius - 1);
+      if (state == GTK_STATE_ACTIVE || state == GTK_STATE_INSENSITIVE)
+      {
+        ge_cairo_set_color (cr, &redmond_style->color_cube.bg[GTK_STATE_NORMAL]);
 
-      inconsistent = (IS_TOGGLE_BUTTON(widget) && gtk_toggle_button_get_inconsistent(TOGGLE_BUTTON(widget)));
-      inconsistent |= (GTK_IS_CELL_RENDERER_TOGGLE(widget) && ge_cell_renderer_toggle_get_inconsistent (widget));
-      inconsistent |= (CHECK_DETAIL(detail, "cellradio") && (shadow == GTK_SHADOW_ETCHED_IN));
- 
-      switch (state)
-	{
-	case GTK_STATE_NORMAL:
-	case GTK_STATE_PRELIGHT:
-          if (inconsistent)
+        cairo_arc  (cr, center_x, center_y,  radius - 2, 0, 2*G_PI);
+
+        cairo_fill (cr);
+      }
+      else
+      {
+        /* Draw stipples for inconsistent checkboxes. */
+        if (shadow == GTK_SHADOW_ETCHED_IN)
           {
-             cairo_t *tmp = ge_gdk_drawable_to_cairo (window, area);
+            cairo_save (cr);
+            cairo_arc  (cr, center_x, center_y,  radius - 2, 0, 2*G_PI);
+            cairo_clip (cr);
 
-            cairo_arc(tmp, center_x, center_y,  radius - 2, 0, 2*M_PI);
-            cairo_clip(tmp);
+            do_redmond_draw_masked_fill (cr, &redmond_style->hatch_mask, 
+                                             &redmond_style->color_cube.bg[GTK_STATE_NORMAL],
+                                             &redmond_style->color_cube.light[GTK_STATE_NORMAL],
+                                             x, y, width, height);
 
-            do_redmond_draw_masked_fill (tmp, &redmond_style->hatch_mask, 
-                                              &redmond_style->color_cube.bg[GTK_STATE_NORMAL],
-                                              &redmond_style->color_cube.light[GTK_STATE_NORMAL],
-                                              x, y, width, height);
-	    cairo_destroy(tmp);
-         }
-          else
-          {
-            ge_cairo_set_color(cr, &redmond_style->color_cube.base[GTK_STATE_NORMAL]);
-            cairo_arc(cr, center_x, center_y, radius - 2, 0, 2 * M_PI);
-            cairo_fill(cr);
+            cairo_restore (cr);
           }
-	  break;
-	default:
-            ge_cairo_set_color(cr, &redmond_style->color_cube.bg[GTK_STATE_NORMAL]);
-            cairo_arc(cr, center_x, center_y, radius - 2, 0, 2 * M_PI);
-            cairo_fill(cr);
-	}
+        else
+          {
+            /* ignore PRELIGHT state ... */
+            ge_cairo_set_color (cr, &redmond_style->color_cube.base[GTK_STATE_NORMAL]);
 
-      if ((shadow == GTK_SHADOW_IN) || inconsistent)
-	{
-           if ((state==GTK_STATE_INSENSITIVE) || inconsistent)
-           {
-             ge_cairo_set_color(cr, &redmond_style->color_cube.fg[GTK_STATE_INSENSITIVE]);
-           }
-           else
-           {
-             ge_cairo_set_color(cr, &redmond_style->color_cube.text[GTK_STATE_NORMAL]);	
-           }
+            cairo_arc  (cr, center_x, center_y,  radius - 2, 0, 2*G_PI);
 
-           cairo_arc(cr, center_x, center_y, radius*0.38, 0, 2 * M_PI);
-           cairo_fill(cr);
-	}
-     }
+            cairo_fill (cr);
+          }
+      }
+
+      if ((shadow == GTK_SHADOW_ETCHED_IN) || (state == GTK_STATE_INSENSITIVE))
+        {
+          /* force insensitive color for inconsistent checkboxes */
+          ge_cairo_set_color(cr, &redmond_style->color_cube.fg[GTK_STATE_INSENSITIVE]);
+          cairo_arc(cr, center_x, center_y, bullet_radius, 0, 2 * G_PI);
+          cairo_fill(cr);
+        }
+      else if (shadow == GTK_SHADOW_IN)
+        {
+          ge_cairo_set_color(cr, &redmond_style->color_cube.text[GTK_STATE_NORMAL]);
+          cairo_arc(cr, center_x, center_y, bullet_radius, 0, 2 * G_PI);
+          cairo_fill(cr);
+        }
+    }
 
      cairo_destroy(cr);
 }
@@ -387,7 +372,7 @@ redmond_draw_arrow (GtkStyle * style,
       || (CHECK_DETAIL (detail, "vscrollbar"))
       || (CHECK_DETAIL (detail, "hscrollbar"))
       || (CHECK_DETAIL (detail, "optionmenu"))
-      || IS_SCROLLBAR(widget)
+      || GE_IS_SCROLLBAR(widget)
       || (ge_is_in_combo_box (widget)))
     {
       if (state != GTK_STATE_INSENSITIVE)
@@ -508,25 +493,23 @@ redmond_draw_shadow (GtkStyle * style,
 	   */
            cairo_set_line_width (cr, 1);
            ge_cairo_set_color(cr, &redmond_style->black_border[state_type]);	
-           cairo_rectangle(cr, x + 0.5, y + 0.5, width - 1, height - 1);
-           cairo_stroke(cr);
+           ge_cairo_stroke_rectangle(cr, x + 0.5, y + 0.5, width - 1, height - 1);
 	}
       else if (((CHECK_DETAIL (detail, "vscrollbar"))
 		|| (CHECK_DETAIL (detail, "hscrollbar"))
 		|| ((CHECK_DETAIL (detail, "button")) && (widget)
-		    && GTK_WIDGET_HAS_DEFAULT (widget))))
+		    && GE_WIDGET_HAS_DEFAULT (widget))))
 	{
 	  /* Scrollbar steppers and Defaultable Buttons use a flat
 	   * dark edge on pressed.
 	   */
            cairo_set_line_width (cr, 1);
            ge_cairo_set_color(cr, &redmond_style->color_cube.dark[state_type]);	
-           cairo_rectangle(cr, x + 0.5, y + 0.5, width - 1, height - 1);
-           cairo_stroke(cr);
+           ge_cairo_stroke_rectangle(cr, x + 0.5, y + 0.5, width - 1, height - 1);
 	}
       else if (((CHECK_DETAIL (detail, "entry"))
 		|| (CHECK_DETAIL (detail, "frame")))
-	       && widget && ((IS_SPIN_BUTTON (widget))
+	       && widget && ((GE_IS_SPIN_BUTTON (widget))
 			     || (ge_is_in_combo_box (widget))))
 	{
 	  /* The Combo/ComboBoxEntry button and the SpingButton Steppers should apear
@@ -556,8 +539,7 @@ redmond_draw_shadow (GtkStyle * style,
 			      height - 2, FALSE);
 	    }
 	}
-      else if ((CHECK_DETAIL (detail, "trough"))
-	       || ((IS_STATUS_BAR (widget)))
+      else if (((GE_IS_STATUSBAR (widget)))
 	       || ((CHECK_DETAIL (detail, "frame")))
 	       || ((CHECK_DETAIL (detail, "button"))
 		   && (ge_is_toolbar_item (widget))))
@@ -607,6 +589,15 @@ redmond_draw_shadow (GtkStyle * style,
 			  &redmond_style->color_cube.dark[state_type], x, y, width, height,
 			  FALSE);
 	}
+      else if (CHECK_DETAIL (detail, "menu"))
+        {
+          ge_cairo_simple_border (cr, &redmond_style->color_cube.bg[state_type],
+                          &redmond_style->black_border[state_type], x, y,
+                          width, height, FALSE);
+          ge_cairo_simple_border (cr, &redmond_style->color_cube.light[state_type],
+                          &redmond_style->color_cube.dark[state_type], x + 1, y + 1, width - 2,
+                          height - 2, FALSE);
+        }
       else
 	{
 	  /* Normal OUT Border */
@@ -687,11 +678,12 @@ redmond_draw_combobox_button (GtkStyle * style,
           gtk_paint_shadow (style, window, state_type, GTK_SHADOW_IN,
   		            area, widget, "entry", x - 2, y, width, height);
         }
+#ifndef GTK_DISABLE_DEPRECATED
       else if (ge_is_combo(widget))
         {
           GtkWidget *entry = widget;
 
-	  if (GTK_IS_WIDGET(widget) && GTK_IS_WIDGET(widget->parent) && GTK_IS_ENTRY(GTK_COMBO (widget->parent)->entry))
+	  if (GE_IS_WIDGET(widget) && GE_IS_WIDGET(widget->parent) && GE_IS_ENTRY(GTK_COMBO (widget->parent)->entry))
             {
                entry = GTK_COMBO (widget->parent)->entry;
                gtk_widget_ensure_style(entry);
@@ -699,7 +691,8 @@ redmond_draw_combobox_button (GtkStyle * style,
                parent_style = entry->style;
                parent_state = entry->state;
              }
-          else if (GTK_IS_WIDGET(widget->parent))
+          else
+                if (GE_IS_WIDGET(widget->parent))
             {
                entry = widget->parent;
                gtk_widget_ensure_style(entry);
@@ -732,6 +725,9 @@ redmond_draw_combobox_button (GtkStyle * style,
 	 	              &clip, entry, "entry", x - 4, y, width + 2, height);
           }
         }
+#else
+#warning Disabling GtkCombo support because GTK_DiSABLE_DEPRECATED is enabled.
+#endif
       else
         {
           GtkWidget *parent = widget;
@@ -759,7 +755,7 @@ redmond_draw_combobox_button (GtkStyle * style,
 
        cr = ge_gdk_drawable_to_cairo (window, area);
  
-        do_redmond_draw_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
+        ge_cairo_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
 				          x, y + 2, width - 2, height - 4);
       cairo_destroy(cr);
 
@@ -789,8 +785,7 @@ redmond_draw_combobox_button (GtkStyle * style,
                 parent_state = GTK_STATE_NORMAL;
 
 	      gdk_draw_rectangle (window,
-		  	          widget->parent->style->
-			          base_gc[GTK_WIDGET_STATE (widget->parent)],
+		  	          parent_style->base_gc[parent_state],
 			          TRUE, x + 2, y, width + 2, height);
             }
           else
@@ -801,11 +796,12 @@ redmond_draw_combobox_button (GtkStyle * style,
           gtk_paint_shadow (style, window, state_type, GTK_SHADOW_IN,
 	  		    area, widget, "entry", x + 2, y, width, height);
         }
+#ifndef GTK_DISABLE_DEPRECATED
       else if (ge_is_combo(widget))
         {
           GtkWidget *entry = widget;
 
-	  if (GTK_IS_WIDGET(widget) && GTK_IS_WIDGET(widget->parent) && GTK_IS_ENTRY(GTK_COMBO (widget->parent)->entry))
+	  if (GE_IS_WIDGET(widget) && GE_IS_WIDGET(widget->parent) && GE_IS_ENTRY(GTK_COMBO (widget->parent)->entry))
             {
                entry = GTK_COMBO (widget->parent)->entry;
                gtk_widget_ensure_style(entry);
@@ -813,7 +809,7 @@ redmond_draw_combobox_button (GtkStyle * style,
                parent_style = entry->style;
                parent_state = entry->state;
              }
-          else if (GTK_IS_WIDGET(widget->parent))
+          else if (GE_IS_WIDGET(widget->parent))
             {
                entry = widget->parent;
                gtk_widget_ensure_style(entry);
@@ -831,6 +827,9 @@ redmond_draw_combobox_button (GtkStyle * style,
            gtk_paint_shadow (parent_style, window, parent_state, GTK_SHADOW_IN,
 			     area, entry, "entry", x + 2, y, width, height);
         }
+#else
+#warning Disabling GtkCombo support because GTK_DiSABLE_DEPRECATED is enabled.
+#endif /* GTK_DISABLE_DEPRECATED */
       else
         {
           GtkWidget *parent = widget;
@@ -858,7 +857,7 @@ redmond_draw_combobox_button (GtkStyle * style,
  
       cr = ge_gdk_drawable_to_cairo (window, area);
 
-      do_redmond_draw_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
+      ge_cairo_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
 				          x + 2, y + 2, width - 2, height - 4);
 
       cairo_destroy(cr);
@@ -924,7 +923,7 @@ redmond_draw_spinbutton_stepper (GtkStyle * style,
 			    &spin_area, widget, detail, x - 2, y,
 			    width + 2, height + 2);
           cr = ge_gdk_drawable_to_cairo (window, area);
-	  do_redmond_draw_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type), 
+	  ge_cairo_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type), 
 					      x, y + 2, width - 2, height - 2);
           cairo_destroy(cr);
 	  gtk_paint_shadow (style, window, state_type, shadow_type,
@@ -940,7 +939,7 @@ redmond_draw_spinbutton_stepper (GtkStyle * style,
 			    &spin_area, widget, detail, x - 2, y - 2,
 			    width + 2, height + 2);
           cr = ge_gdk_drawable_to_cairo (window, area);
-	  do_redmond_draw_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
+	  ge_cairo_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
 					      x, y, width - 2, height - 2);
           cairo_destroy(cr);
 	  gtk_paint_shadow (style, window, state_type, shadow_type,
@@ -959,7 +958,7 @@ redmond_draw_spinbutton_stepper (GtkStyle * style,
 			    &spin_area, widget, detail, x, y,
 			    width + 4, height + 2);
           cr = ge_gdk_drawable_to_cairo (window, area);
-	  do_redmond_draw_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type), 
+	  ge_cairo_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type), 
 					      x + 2, y + 2, width - 2, height - 2);
           cairo_destroy(cr);
 	  gtk_paint_shadow (style, window, state_type, shadow_type,
@@ -975,7 +974,7 @@ redmond_draw_spinbutton_stepper (GtkStyle * style,
 			    &spin_area, widget, detail, x, y - 2,
 			    width + 4, height + 2);
           cr = ge_gdk_drawable_to_cairo (window, area);
-	  do_redmond_draw_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type), 
+	  ge_cairo_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type), 
 					      x + 2, y, width - 2, height - 2);
           cairo_destroy(cr);
 	  gtk_paint_shadow (style, window, state_type, shadow_type,
@@ -1017,13 +1016,13 @@ redmond_draw_box (GtkStyle * style,
   CHECK_ARGS
   SANITIZE_SIZE
  
-  if (IS_MENU_SHELL(widget))
+  if (GE_IS_MENU_SHELL(widget))
     {
       redmond_gtk2_engine_hack_menu_shell_setup_signals(widget);
     }  
  
   if ((CHECK_DETAIL (detail, "trough")) &&
-      (widget && IS_SCROLLBAR (widget)))
+      (widget && GE_IS_SCROLLBAR (widget)))
     {
       /* Scrollbar Troughs have a thin cross-hatch 
        * background fill. Technically its supposed
@@ -1036,13 +1035,11 @@ redmond_draw_box (GtkStyle * style,
                                          &redmond_style->color_cube.light[state_type],
                                          x, y, width, height);
  
-           cairo_destroy(cr);			          
+      cairo_destroy(cr);
 
-      redmond_draw_shadow (style, window, state_type, GTK_SHADOW_IN, area,
-			widget, detail, x, y, width, height);
     }
   else if ((CHECK_DETAIL (detail, "trough"))
-	   && (widget && IS_SCALE (widget)))
+	   && (widget && GE_IS_SCALE (widget)))
     {
       /* Scale Troughs are only 4 pixels thick
        * i.e. just a border, so make sure the background
@@ -1050,9 +1047,9 @@ redmond_draw_box (GtkStyle * style,
        */
       cr = ge_gdk_drawable_to_cairo (window, area);
 
-      do_redmond_draw_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
+      ge_cairo_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, GTK_STATE_NORMAL),
                                     x, y, width, height);
-      if (IS_HSCALE (widget))
+      if (GE_IS_HSCALE (widget))
 	{
           ge_cairo_set_color(cr, &redmond_style->black_border[state_type]);
           cairo_rectangle(cr, x, y + (height / 2), width - 2, 1);
@@ -1076,7 +1073,7 @@ redmond_draw_box (GtkStyle * style,
     }
   else if ((CHECK_DETAIL (detail, "toolbar"))
 	   || (CHECK_DETAIL (detail, "menubar"))
-	   || (IS_BONOBO_TOOLBAR (widget))
+	   || (GE_IS_BONOBO_TOOLBAR (widget))
 	   || (CHECK_DETAIL (detail, "dockitem"))
 	   || (CHECK_DETAIL (detail, "dockitem_bin"))
 	   || (CHECK_DETAIL (detail, "handlebox_bin"))
@@ -1090,17 +1087,17 @@ redmond_draw_box (GtkStyle * style,
                 top_cutoff = FALSE, bottom_cutoff = FALSE;
  
       if (((CHECK_DETAIL (detail, "dockitem_bin")) && 
-           (IS_BONOBO_DOCK_ITEM(widget))) || 
+           (GE_IS_BONOBO_DOCK_ITEM(widget))) || 
           ((widget) && (ge_is_bonobo_dock_item(widget->parent))))
 	{	  
 	  GList *children = NULL, *child = NULL;
 	  GtkWidget *dockitem = widget;
 	  gboolean has_grip = FALSE, ltr = TRUE;
 	  
-	  if ((!IS_BONOBO_DOCK_ITEM(widget)) && (!IS_BOX(widget)))
+	  if ((!GE_IS_BONOBO_DOCK_ITEM(widget)) && (!GE_IS_BOX(widget)))
 	    dockitem = widget->parent;
 	    
-	  has_grip = IS_CONTAINER(dockitem);
+	  has_grip = GE_IS_CONTAINER(dockitem);
 	  
 	  ltr = (!widget) || (gtk_widget_get_direction (dockitem) == GTK_TEXT_DIR_LTR);
 	  
@@ -1112,7 +1109,7 @@ redmond_draw_box (GtkStyle * style,
                           
             for (child = g_list_first(children); child; child = g_list_next(child))
               {
-	        if (IS_BONOBO_DOCK_ITEM_GRIP(child->data))
+	        if (GE_IS_BONOBO_DOCK_ITEM_GRIP(child->data))
                   has_grip = (GTK_WIDGET_VISIBLE(child->data) && 
                               GTK_WIDGET_REALIZED(child->data) && 
                               GTK_WIDGET(child->data)->allocation.width > 1) &&
@@ -1137,9 +1134,9 @@ redmond_draw_box (GtkStyle * style,
 	       
    	      GtkOrientation tmp = GTK_ORIENTATION_HORIZONTAL;
  
-	      if IS_HBOX(dockitem)
+	      if GE_IS_HBOX(dockitem)
 	        tmp = GTK_ORIENTATION_VERTICAL;
-	      else if (IS_BONOBO_DOCK_ITEM(dockitem))
+	      else if (GE_IS_BONOBO_DOCK_ITEM(dockitem))
 	        g_object_get (dockitem, "orientation", &tmp, NULL);
  
 	      if (tmp == GTK_ORIENTATION_HORIZONTAL)
@@ -1174,7 +1171,7 @@ redmond_draw_box (GtkStyle * style,
 	        }
 	    }    
 	}
-      else if (IS_HANDLE_BOX(widget))
+      else if (GE_IS_HANDLE_BOX(widget))
         {
 	  switch (gtk_handle_box_get_handle_position
 		  (GTK_HANDLE_BOX (widget)))
@@ -1198,7 +1195,7 @@ redmond_draw_box (GtkStyle * style,
 	      break;
 	    }      
         }
-      else if (IS_HANDLE_BOX_ITEM(widget) && GTK_WIDGET_REALIZED(widget->parent) && GTK_WIDGET_VISIBLE(widget->parent))
+      else if (GE_IS_HANDLE_BOX_ITEM(widget) && GTK_WIDGET_REALIZED(widget->parent) && GTK_WIDGET_VISIBLE(widget->parent))
         {
 	  switch (gtk_handle_box_get_handle_position
 		  (GTK_HANDLE_BOX (widget->parent)))
@@ -1237,7 +1234,7 @@ redmond_draw_box (GtkStyle * style,
         }
   
       cr = ge_gdk_drawable_to_cairo (window, area);
-      do_redmond_draw_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
+      ge_cairo_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
 					  x, y, width, height);
  
       /* If this is a menu embedded in the gnome-panel, we don't
@@ -1264,7 +1261,7 @@ redmond_draw_box (GtkStyle * style,
       cairo_destroy(cr);
     }
   else if ((CHECK_DETAIL (detail, "menuitem")) && widget && widget->parent
-	   && IS_MENU_BAR (widget->parent))
+	   && GE_IS_MENU_BAR (widget->parent))
     {
       /* Primary Menu Items on Menu bars are drawn with 
        * a thin inset border on select/active,
@@ -1277,10 +1274,10 @@ redmond_draw_box (GtkStyle * style,
  
       cr = ge_gdk_drawable_to_cairo (window, area);
 
-      do_redmond_draw_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
+      ge_cairo_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
 					  x, y, width, height);
 		 			  
-      if ((!GTK_IS_MENU(GTK_MENU_ITEM(widget)->submenu)) || 
+      if ((!GE_IS_MENU(GTK_MENU_ITEM(widget)->submenu)) || 
           (!(GTK_WIDGET_REALIZED(GTK_MENU_ITEM(widget)->submenu) && 
              GTK_WIDGET_VISIBLE(GTK_MENU_ITEM(widget)->submenu) &&
              GTK_WIDGET_REALIZED(GTK_MENU(GTK_MENU_ITEM(widget)->submenu)->toplevel) &&
@@ -1302,7 +1299,7 @@ redmond_draw_box (GtkStyle * style,
       cairo_destroy(cr);
     }
   else if ((CHECK_DETAIL (detail, "bar"))
-	   && (widget && IS_PROGRESS_BAR (widget)))
+	   && (widget && GE_IS_PROGRESS_BAR (widget)))
     {
       /* Progress bar status is always flat, and only has two states
        * NORMAL and INSENSITIVE, so check the state, and then do a 
@@ -1312,7 +1309,7 @@ redmond_draw_box (GtkStyle * style,
 	state_type = GTK_STATE_NORMAL;
 
       cr = ge_gdk_drawable_to_cairo (window, area);
-      do_redmond_draw_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, GTK_STATE_SELECTED),
+      ge_cairo_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, GTK_STATE_SELECTED),
                                     x, y, width, height);
       cairo_destroy(cr);
     }
@@ -1334,7 +1331,7 @@ redmond_draw_box (GtkStyle * style,
       redmond_draw_spinbutton_stepper (style, window, state_type, shadow_type, area,
 			       widget, detail, x, y, width, height);
     }
-  else if (IS_TOGGLE_BUTTON(widget) && (TOGGLE_BUTTON(widget)->active))
+  else if (GE_IS_TOGGLE_BUTTON (widget) && (shadow_type == GTK_SHADOW_IN))
     {
       gint pointer_x, pointer_y;
       GdkModifierType pointer_mask;
@@ -1349,7 +1346,7 @@ redmond_draw_box (GtkStyle * style,
 	  (pointer_y < (widget->allocation.y +
 	                widget->allocation.height)))
         {
-          do_redmond_draw_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
+          ge_cairo_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
                                         x, y, width, height);
         }
       else 
@@ -1369,7 +1366,7 @@ redmond_draw_box (GtkStyle * style,
       /* default box apearance */
       cr = ge_gdk_drawable_to_cairo (window, area);
 
-      do_redmond_draw_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
+      ge_cairo_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
                                     x, y, width,height);
       cairo_destroy(cr);
       redmond_draw_shadow (style, window, state_type, shadow_type, area,
@@ -1465,7 +1462,7 @@ redmond_draw_slider (GtkStyle * style,
   if ((CHECK_DETAIL (detail, "hscale")) || (CHECK_DETAIL (detail, "vscale")))
     {
       cairo_t *cr = ge_gdk_drawable_to_cairo (window, area);
-      do_redmond_draw_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
+      ge_cairo_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
                                     x, y, width, height);
  
       cairo_destroy(cr);
@@ -1508,16 +1505,37 @@ redmond_draw_extension (GtkStyle * style,
   CairoColor *color3 = NULL;
   CairoColor *color4 = NULL;
   cairo_t *cr;
+  gint tab_overlap = 0;
  
   CHECK_ARGS
   SANITIZE_SIZE
- 
+
+  if (GE_IS_NOTEBOOK (widget))
+    gtk_widget_style_get (widget, "tab-overlap", &tab_overlap, NULL);
+
+  /* If the tab_overlap is large, then decrease the size of active tabs. */
+  if (state_type == GTK_STATE_ACTIVE && tab_overlap >= 4)
+    {
+      if (gap_side == GTK_POS_TOP || gap_side == GTK_POS_BOTTOM)
+        {
+          x += 2;
+          width -= 4;
+        }
+      else
+        {
+          y += 2;
+          height -= 4;
+        }
+    }
+
   cr = ge_gdk_drawable_to_cairo (window, area);
-  do_redmond_draw_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, GTK_STATE_NORMAL),
+  ge_cairo_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, GTK_STATE_NORMAL),
                                 x, y, width, height);
   switch (shadow_type)
     {
     case GTK_SHADOW_NONE:
+      /* need to destroy the cairo context */
+      cairo_destroy (cr);
       return;
     case GTK_SHADOW_IN:
       color1 = &redmond_style->color_cube.dark[state_type];
@@ -1555,7 +1573,7 @@ redmond_draw_extension (GtkStyle * style,
       switch (gap_side)
 	{
 	case GTK_POS_TOP:
-	  do_redmond_draw_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
+	  ge_cairo_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
                                         x + style->xthickness, 
                                         y,
                                         width - (2 * style->xthickness),
@@ -1575,7 +1593,7 @@ redmond_draw_extension (GtkStyle * style,
 			 y + height - 2);
 	  break;
 	case GTK_POS_BOTTOM:
-	  do_redmond_draw_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
+	  ge_cairo_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
                                         x + style->xthickness, 
                                         y + style->ythickness,
                                         width - (2 * style->xthickness),
@@ -1596,7 +1614,7 @@ redmond_draw_extension (GtkStyle * style,
 			 y + 1);
 	  break;
 	case GTK_POS_LEFT:
-	  do_redmond_draw_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
+	  ge_cairo_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
                                         x,
                                         y + style->ythickness,
                                         width - (style->xthickness),
@@ -1615,7 +1633,7 @@ redmond_draw_extension (GtkStyle * style,
 			 y + height - 2);
 	  break;
 	case GTK_POS_RIGHT:
-	  do_redmond_draw_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
+	  ge_cairo_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type),
                                         x + style->xthickness,
                                         y + style->ythickness,
                                         width - (style->xthickness),
@@ -1666,9 +1684,10 @@ redmond_draw_handle (GtkStyle * style,
   CHECK_ARGS
   SANITIZE_SIZE
   
-  if (IS_BONOBO_DOCK_ITEM_GRIP(widget) && 
+  if (GE_IS_BONOBO_DOCK_ITEM_GRIP(widget) && 
      (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL) && 
-      orientation == (GTK_ORIENTATION_HORIZONTAL))
+     (orientation == (GTK_ORIENTATION_HORIZONTAL)) &&
+     (widget->parent != NULL))
   {
     x = widget->parent->allocation.width - widget->allocation.width;
     y = widget->parent->allocation.height - widget->allocation.height;
@@ -1679,12 +1698,12 @@ redmond_draw_handle (GtkStyle * style,
   }
   
   cr = ge_gdk_drawable_to_cairo (window, area);
-  do_redmond_draw_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type), 
+  ge_cairo_pattern_fill (cr, DEFAULT_BACKGROUND_PATTERN(redmond_style, state_type), 
                                 x, y, width, height);
  
-  if (widget && !(IS_PANED (widget)))
+  if (widget && !(GE_IS_PANED (widget)))
     {
-      if (IS_HANDLE_BOX (widget))
+      if (GE_IS_HANDLE_BOX (widget))
 	{
 	  /* handle box apears to be broken in that
 	   * it doesn't pass the orientation properly,
@@ -1708,7 +1727,7 @@ redmond_draw_handle (GtkStyle * style,
       else
 	{
 	  if ((CHECK_DETAIL (detail, "handlebox")
-	       && (!IS_HANDLE_BOX_ITEM (widget))))
+	       && (!GE_IS_HANDLE_BOX_ITEM (widget))))
 	    {
 	      /* panel_applet_frame used by the panel is broken,
 	       * always passes orientation == horizontal 
@@ -1759,8 +1778,8 @@ redmond_draw_handle (GtkStyle * style,
 
       if (ge_is_panel_widget_item (widget)
 	  && (CHECK_DETAIL (detail, "handlebox")
-	      && (!IS_HANDLE_BOX_ITEM (widget)))
-	  && (!(IS_HANDLE_BOX (widget))))
+	      && (!GE_IS_HANDLE_BOX_ITEM (widget)))
+	  && (!(GE_IS_HANDLE_BOX (widget))))
 	{
 	  /* If this is on a PanelWidget, we draw a line 
 	   * next to it instead of drawing a border around it.
@@ -1795,9 +1814,9 @@ redmond_draw_handle (GtkStyle * style,
           else
             clip = shadow;
  
-          if (IS_BONOBO_DOCK_ITEM_GRIP(widget))
+          if (GE_IS_BONOBO_DOCK_ITEM_GRIP(widget))
             {
-              if IS_BOX(widget->parent)
+              if GE_IS_BOX(widget->parent)
                 {
                   GList *children = NULL, *child = NULL;
  
@@ -1805,7 +1824,7 @@ redmond_draw_handle (GtkStyle * style,
               
                   for (child = g_list_first(children); child; child = g_list_next(child))
                     {
-	              if (IS_BOX(child->data))
+	              if (GE_IS_BOX(child->data))
 	                {
 	                  skip_shadow = TRUE;
 	                  child = NULL;
